#================================================
#
#    xml_processing.py - Wallpapoz 
#    Copyright (C) 2005 Akbar <akbarhome@gmail.com>
#
#================================================
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#================================================

## xml_processing.py
#
# This file manipulate xml file for gui or daemon

from xml.dom import minidom
import xml
import os
import sys

## XMLProcessing -- class for processing wallpapoz xml file
class XMLProcessing:

    ## The constructor
    def __init__(self, file):
	if not os.path.exists(file):
	    print "No configuration file. Use default configuration."
	    home = os.environ['HOME']
	    if not os.path.exists(home + '/.wallpapoz'):
		os.makedirs(home + '/.wallpapoz')
	    current_wallpaper = os.popen("gconftool-2 -g /desktop/gnome/background/picture_filename").read()[:-1]
	    number_workspaces = int(os.popen("xprop -root _NET_NUMBER_OF_DESKTOPS").read()[36:38])
	    default_list = []
	    for i in range(number_workspaces):
		default_list.append([])
		default_list[i].append("rename this")
		default_list[i].append(current_wallpaper)
	    self.create_configuration_file(file, default_list)

        try:
	    self.xmldoc = minidom.parse(file)
	except xml.parsers.expat.ExpatError:
	    print "The configuration file is corrupted. Remove it then create it again with wallpapoz!"
	    sys.exit()

	# wallpapoz node
	self.wallpapoz_node = self.xmldoc.childNodes[1]

        # workspace node list
	self.workspace_node_list = self.wallpapoz_node.getElementsByTagName('workspace')

	# amount of workspace node
	self.workspace_num = self.workspace_node_list.length

    ## class method -- get delay time that wallpaper list thread manipulate its index
    def delay(self):
	return self.wallpapoz_node.attributes["interval"].value

    ## class method -- whether the wallpaper list thread randomize its index
    def is_random(self):
	return self.wallpapoz_node.attributes["random"].value

    ## class method -- set delay time
    def set_delay(self, time):
	self.wallpapoz_node.setAttribute('interval', time )

    ## class method -- set random
    def set_random(self, randm):
	self.wallpapoz_node.setAttribute('random', randm )

    ## class method -- method for fill list with wallpaper path file
    def fill_list(self):

	# container for wallpapers list
	worklist = []
	for i in range(self.workspace_num):
	    worklist.append( [] )
	index = -1

        # for every workspace
	for file in self.workspace_node_list:
	    file_node_list = file.getElementsByTagName('file')
	    index = index + 1
	    worklist[index].append( file.attributes["name"].value )
	    for node in file_node_list:
		if node.nodeName == "file":
		    worklist[index].append(node.firstChild.data)

	return worklist

    ## class method -- to know how many workspace we use
    def get_workspace_num(self):
	return self.workspace_num

    ## class method -- create configuration file
    def create_configuration_file(self, config_file, wallpaperlist, delay=5, random=0):

	# use minidom implementation
	impl = minidom.getDOMImplementation()

        # this is the documenttype and document
	doctype = impl.createDocumentType("Wallpapoz", None, None)
	newdoc = impl.createDocument(None, "wallpapoz", doctype)

	# root element ( wallpapoz )
	top_element = newdoc.documentElement

	# pretty printing
	space_wallpapoz = newdoc.createTextNode("\n")
	top_element.appendChild(space_wallpapoz)
	# end of pretty printing

	# put attributes to wallpapoz element
	top_element.setAttribute('random', str(random) )
	top_element.setAttribute('interval', str(delay) )

        index_of_wallpaperlist = 1

	# create workspace node based on wallpaper list
	for workspace in wallpaperlist:
	    workspaceelement = newdoc.createElement("workspace")
	    workspaceelement.setAttribute("no", str(index_of_wallpaperlist) )
	    workspaceelement.setAttribute("name", workspace.pop(0) )

	    # add the index
	    index_of_wallpaperlist += 1

	    # pretty printing
	    space_workspace_tab = newdoc.createTextNode("    ")
	    top_element.appendChild(space_workspace_tab)
	    space_workspace = newdoc.createTextNode("\n")
	    workspaceelement.appendChild(space_workspace)
	    # end of pretty printing

	    for file in workspace:

		# pretty printing
		space_file_tab = newdoc.createTextNode("        ")
		workspaceelement.appendChild(space_file_tab)
		# end of pretty printing

		fileelement = newdoc.createElement("file")
		path = newdoc.createTextNode(file)
		fileelement.appendChild(path)
		workspaceelement.appendChild(fileelement)

		# pretty printing
		space_file = newdoc.createTextNode("\n")
		workspaceelement.appendChild(space_file)
		# end of pretty printing 

	    # pretty printing
	    space_file_tab = newdoc.createTextNode("    ")
	    workspaceelement.appendChild(space_file_tab)
	    # end of pretty printing

	    top_element.appendChild(workspaceelement)

	    # pretty printing
	    space = newdoc.createTextNode("\n")
	    top_element.appendChild(space)
	    # end of pretty printing

        # create it
	xml_file = open(config_file, "w")
	xml_file.write( newdoc.toxml("utf-8") )
